use crate::cli::Issue2Work;
use crate::config::Config;
use crate::error::GeneralError;
use crate::gitlab::action::GitlabAction;
use crate::planning::Issue2WorkActionTrait;
use url::Url;

struct App {
    gitlab_issue2work_action: GitlabAction,
}

trait Issue2WorkPackageAction {
    fn issue2work_package_action(
        &self,
        config: &Config,
        args: &Issue2Work,
    ) -> Result<(), GeneralError>;
}

pub(crate) async fn issue2work(config: Config, args: &Issue2Work) -> Result<(), GeneralError> {
    let url = Url::parse(&*args.issue_url).expect("issue_url is not valid");
    let app = App {
        gitlab_issue2work_action: GitlabAction {},
    };

    if (app
        .gitlab_issue2work_action
        .supports(&url, &config, args)
        )
    {
        app.gitlab_issue2work_action.run(&url, &config, args).await
    } else {
        Err(GeneralError {
            description: format!("This action is not supported for this url: {}", url),
        })
    }
}
